var pluginInfo = { "Name":"Android Profile Validator", "Description":"Generates an Android application with which to validate the profile on a connected device", "Author":"Martin Woolley", "Version":"1.0.1", "IsClient":true };

function GetInfo(infoObject) {
    infoObject.Name = pluginInfo.Name;
    infoObject.Description = pluginInfo.Description;
    infoObject.Author = pluginInfo.Author;
    infoObject.Version = pluginInfo.Version;
    infoObject.IsClient = pluginInfo.IsClient;
    return infoObject;
} 

function RunPlugin(profiledata) {
    log("Starting ...");  

	var namespace_reqd = profiledata.CustomNameSpace;

	var namespace_as_dir = namespace_reqd.replace(/\./g,"\\");
	var rx_custom_namespace_tag = new RegExp("{{CustomNameSpace}}","g");
	
    var template = FileManager.ReadFile("Constants.java.template");
    var result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("Constants.java", result);

	// Start Template plus JavaScript Derived String plus End Template

	// read start template
    template = FileManager.ReadFile("ExpectedProfileFactory.java.template.start");
    var epf = ProcessTemplate(template, profiledata);
	
	
	// Add to the epf variable using JavaScript to extract data from the profile object model
    for (var x = 0; x < profiledata.Services.length; x++) {
        var service = profiledata.Services[x];
		var svc_id = 's'+x;
        epf = epf + '        BleService '+svc_id;
		epf = epf +' = new BleService("';
		epf = epf + service.UUID+'","'+service.Name+'");\r\n';
		epf = epf + '        expected_profile.addService('+svc_id+');\r\n';

        for (var y = 0; y < service.Characteristics.length; y++) {
            var characteristic = service.Characteristics[y];
		    var char_id = svc_id + '_' + 'c'+y;
	        epf = epf + '        BleCharacteristic '+char_id + '= new BleCharacteristic("';
			epf = epf + characteristic.UUID+'","';
			epf = epf + characteristic.Name+'");\r\n';
		    epf = epf + '        ' + svc_id + '.addCharacteristic('+char_id+');\r\n';
			for (var z = 0; z < characteristic.Descriptors.length; z++) {
				var descriptor = characteristic.Descriptors[z];
				var desc_id = char_id + '_' + 'd'+z;
				epf = epf + '        BleDescriptor '+desc_id + '= new BleDescriptor("';
				epf = epf + descriptor.UUID+'","';
				epf = epf + descriptor.Name+'");\r\n';
				epf = epf + '        ' + char_id + '.addDescriptor('+desc_id+');\r\n';
			}

			// properties - only one set supported
			epf = epf + '        properties = 0;\r\n';
			if (characteristic.Properties[0].Broadcast == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_BROADCAST;\r\n';
			}
			if (characteristic.Properties[0].Indicate == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_INDICATE;\r\n';
			}
			if (characteristic.Properties[0].Notify == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_NOTIFY;\r\n';
			}
			if (characteristic.Properties[0].Read == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_READ;\r\n';
			}
			if (characteristic.Properties[0].ReliableWrite == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_RELIABLE_WRITE;\r\n';
			}
			if (characteristic.Properties[0].SignedWrite == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_SIGNED_WRITE;\r\n';
			}
			if (characteristic.Properties[0].Write == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_WRITE;\r\n';
			}
			if (characteristic.Properties[0].WriteWithoutResponse == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE;\r\n';
			}
			if (characteristic.Properties[0].ExtendedProperties == 'Mandatory') {
			    epf = epf + '        properties = properties | BluetoothGattCharacteristic.PROPERTY_EXTENDED_PROPERTIES;\r\n';
			}	
			epf = epf + '        ' + char_id + '.setProperties(properties);\r\n';
		}
	}

	// read end template
    template = FileManager.ReadFile("ExpectedProfileFactory.java.template.end");
	// and add to the epf variable
    epf = epf + ProcessTemplate(template, profiledata);	

    // Output the epf variable to a file
    FileManager.CreateFile("ExpectedProfileFactory.java", epf);
	
    template = FileManager.ReadFile("activity_log.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("activity_log.xml", result);

    template = FileManager.ReadFile("activity_main.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("activity_main.xml", result);

    template = FileManager.ReadFile("AndroidManifest.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("AndroidManifest.xml", result);

    template = FileManager.ReadFile("BleAdapterService.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleAdapterService.java", result);
	
    template = FileManager.ReadFile("BleAttribute.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleAttribute.java", result);

    template = FileManager.ReadFile("BleCharacteristic.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleCharacteristic.java", result);

    template = FileManager.ReadFile("BleDescriptor.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleDescriptor.java", result);
	
    template = FileManager.ReadFile("BleProfile.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleProfile.java", result);

    template = FileManager.ReadFile("BleService.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("BleService.java", result);

    template = FileManager.ReadFile("Handle.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("Handle.java", result);

    template = FileManager.ReadFile("list_row.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("list_row.xml", result);

    template = FileManager.ReadFile("MainActivity.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("MainActivity.java", result);

    template = FileManager.ReadFile("ProfileTestActivity.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("ProfileTestActivity.java", result);
	
    template = FileManager.ReadFile("strings.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("strings.xml", result);

    template = FileManager.ReadFile("styles.xml.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("styles.xml", result);

    template = FileManager.ReadFile("TestLog.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("TestLog.java", result);

    template = FileManager.ReadFile("TestResultUser.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("TestResultUser.java", result);

    template = FileManager.ReadFile("Utility.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("Utility.java", result);

    template = FileManager.ReadFile("CharacteristicCheckResult.java.template");
    result = ProcessTemplate(template, profiledata);
    FileManager.CreateFile("CharacteristicCheckResult.java", result);

    var copyToAndroidStudioTemplate = FileManager.ReadFile("copy_to_android_studio.bat.template");
	var copyToAndroidStudio = copyToAndroidStudioTemplate.replace(rx_custom_namespace_tag, namespace_as_dir);
    FileManager.CreateFile("copy_to_android_studio.bat", copyToAndroidStudio);
	
    log("Finished");
}
